package pucrs.hslcommunicationservice.aws;

import com.amazonaws.auth.AWSStaticCredentialsProvider;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.AmazonS3ClientBuilder;
import com.amazonaws.services.s3.model.PutObjectResult;
import com.amazonaws.util.IOUtils;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;


@Component
public class S3Client extends Credentials {

    private final AmazonS3 s3client;

    private final String bucketName = "comunicahsl";

    public S3Client() {
        this.s3client = AmazonS3ClientBuilder.standard()
                .withRegion("us-east-2")
                .withCredentials(new AWSStaticCredentialsProvider(setup()))
                .build();
    }

    public PutObjectResult uploadAttachment(String fileName, String base64Content) {
        return s3client.putObject(bucketName, fileName, base64Content);
    }

    public File getAttachmentAsFile(String fileName) throws IOException {
        File file = new File(fileName);
        IOUtils.copy(s3client.getObject(bucketName, fileName).getObjectContent(), new FileOutputStream(file));
        return file;
    }

    public String getAttachmentAsBase64(String fileName) throws IOException {
        return s3client.getObjectAsString(bucketName, fileName);
    }

    public void removeAttachment(String fileName) {
        s3client.deleteObject(bucketName, fileName);
    }

}
