package pucrs.hslcommunicationservice.service;

import com.opencsv.CSVReader;
import com.opencsv.CSVReaderBuilder;
import com.opencsv.exceptions.CsvException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pucrs.hslcommunicationservice.aws.S3Client;
import pucrs.hslcommunicationservice.domain.User;
import pucrs.hslcommunicationservice.repository.UserRepository;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.time.LocalDate;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class UserService {

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private S3Client s3Client;

    @PersistenceContext
    private EntityManager entityManager;

    public User addUser(User user) {
        return userRepository.save(user);
    }

    public List<User> getUsers() {
        return userRepository.findAll();
    }

    public User getUserByRegistry(Long registry) {
        return userRepository.getById(registry);
    }

    public boolean deleteUser(Long registry) {
        userRepository.delete(getUserByRegistry(registry));
        return true;
    }

    public List<User> importUsers(String csvFromS3) throws IOException {
        File csv = s3Client.getAttachmentAsFile(csvFromS3);
        try {
            CSVReader reader = new CSVReaderBuilder(new FileReader(csv))
                    .withSkipLines(1)
                    .build();

            return reader.readAll()
                    .stream()
                    .map(data -> userRepository.save(buildUser(data)))
                    .collect(Collectors.toList());
        } catch (IOException | CsvException e) {
            e.printStackTrace();
        }
        return Collections.emptyList();
    }

    private User buildUser(String[] data) {
        int day = Integer.parseInt(data[2].split("/")[0]);
        int month = Integer.parseInt(data[2].split("/")[1]);

        return User.builder()
                .registry(Long.parseLong(data[0]))
                .name(data[1])
                .day(day)
                .month(month)
                .occupation(data[4])
                .isAdmin(false)
                .department(departmentService.getDepartamentByName(data[5]))
                .build();
    }

    public List<User> getBirthdayUsers(int month) {
        int today = LocalDate.now().getDayOfMonth();
        String monthFilter = "select u from User u where u.month =: month";
        String todayFilter = monthFilter + " and u.day >= :today";

        return month == LocalDate.now().getMonth().getValue() ?
                entityManager.createQuery(todayFilter, User.class)
                .setParameter("month", month)
                .setParameter("today", today)
                .getResultList() :
                entityManager.createQuery(monthFilter, User.class)
                .setParameter("month", month)
                .getResultList();
    }
}